@@ -5,11 +5,11 @@ module Main where
 import Lens.Micro ((^.), (&), (.~), (%~))
 import Lens.Micro.TH (makeLenses)
 import Control.Monad (void, forever)
-import Control.Concurrent (newChan, writeChan, threadDelay, forkIO)
-import Data.Default
+import Control.Concurrent (threadDelay, forkIO)
 import Data.Monoid
 import qualified Graphics.Vty as V
 
+import Brick.BChan
 import Brick.Main
   ( App(..)
   , showFirstCursor
@@ -17,43 +17,48 @@ import Brick.Main
   , continue
   , halt
   )
+import Brick.AttrMap
+  ( attrMap
+  )
 import Brick.Types
   ( Widget
   , Next
   , EventM
+  , BrickEvent(..)
   )
 import Brick.Widgets.Core
   ( (<=>)
   , str
   )
 
+data CustomEvent = Counter deriving Show
+
 data St =
-    St { _stLastVtyEvent :: Maybe V.Event
+    St { _stLastBrickEvent :: Maybe (BrickEvent () CustomEvent)
        , _stCounter :: Int
        }
 
 makeLenses ''St
 
-data CustomEvent = VtyEvent V.Event
-                 | Counter
-
 drawUI :: St -> [Widget ()]
 drawUI st = [a]
     where
-        a = (str $ "Last Vty event: " <> (show $ st^.stLastVtyEvent))
+        a = (str $ "Last event: " <> (show $ st^.stLastBrickEvent))
             <=>
             (str $ "Counter value is: " <> (show $ st^.stCounter))
 
-appEvent :: St -> CustomEvent -> EventM () (Next St)
+appEvent :: St -> BrickEvent () CustomEvent -> EventM () (Next St)
 appEvent st e =
     case e of
         VtyEvent (V.EvKey V.KEsc []) -> halt st
-        VtyEvent ev -> continue $ st & stLastVtyEvent .~ (Just ev)
-        Counter -> continue $ st & stCounter %~ (+1)
+        VtyEvent _ -> continue $ st & stLastBrickEvent .~ (Just e)
+        AppEvent Counter -> continue $ st & stCounter %~ (+1)
+                                          & stLastBrickEvent .~ (Just e)
+        _ -> continue st
 
 initialState :: St
 initialState =
-    St { _stLastVtyEvent = Nothing
+    St { _stLastBrickEvent = Nothing
        , _stCounter = 0
        }
 
@@ -63,16 +68,15 @@ theApp =
         , appChooseCursor = showFirstCursor
         , appHandleEvent = appEvent
         , appStartEvent = return
-        , appAttrMap = def
-        , appLiftVtyEvent = VtyEvent
+        , appAttrMap = const $ attrMap V.defAttr []
         }
 
 main :: IO ()
 main = do
-    chan <- newChan
+    chan <- newBChan 10
 
     forkIO $ forever $ do
-        writeChan chan Counter
+        writeBChan chan Counter
         threadDelay 1000000
 
-    void $ customMain (V.mkVty def) chan theApp initialState
+    void $ customMain (V.mkVty V.defaultConfig) (Just chan) theApp initialState
